'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = formReducer;

var _formConstants = require('./formConstants');

var c = _interopRequireWildcard(_formConstants);

var _createDefaultFields = require('./utils/createDefaultFields');

var _createDefaultForm = require('./utils/createDefaultForm');

var _createDefaultForm2 = _interopRequireDefault(_createDefaultForm);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Form reducer
 *
 * @param {Object} state  - The current state
 * @param {Object} action - The action to process
 *
 * @return {Object} - The new state
 */
function formReducer() {
	var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
	var action = arguments[1];

	/**
  * FORM_INIT
  *
  * Initialise a new form's state
  *
  * @param {String} action.formName
  * @param {Object} action.fields
  * @param {Object} action.values
  */
	if (action.type === c.FORM_INIT) {
		return _extends({}, state, _defineProperty({}, action.formName, (0, _createDefaultForm2.default)(action.fields, action.values)));
	}

	/**
  * FORM_ADD_FIELD
  *
  * Add a new field to the form
  *
  * @param {String} action.formName
  * @param {String} action.fieldName
  * @param {Mixed} action.value
  */
	if (action.type === c.FORM_ADD_FIELD) {
		var value = typeof action.value !== 'undefined' ? action.value : '';

		return _extends({}, state, _defineProperty({}, action.formName, _extends({}, state[action.formName], {
			fields: _extends({}, state[action.formName].fields, _defineProperty({}, action.fieldName, _extends({}, _createDefaultFields.initialFieldState))),
			initial: _extends({}, state[action.formName].values, _defineProperty({}, action.fieldName, value)),
			values: _extends({}, state[action.formName].values, _defineProperty({}, action.fieldName, value))
		})));
	}

	/**
  * FORM_RESET
  *
  * Reset a form's values to its initial values
  */
	if (action.type === c.FORM_RESET) {
		return _extends({}, state, _defineProperty({}, action.formName, _extends({}, state[action.formName], {
			values: _extends({}, state[action.formName].initial)
		})));
	}

	/**
  * FIELD_UPDATE_VALUE
  *
  * Update the value of a field
  *
  * @param {String} action.formName
  * @param {String} action.fieldName
  * @param {Mixed} action.value
  */
	if (action.type === c.FIELD_UPDATE_VALUE) {
		return _extends({}, state, _defineProperty({}, action.formName, _extends({}, state[action.formName], {
			fields: _extends({}, state[action.formName].fields, _defineProperty({}, action.fieldName, _extends({}, state[action.formName].fields[action.fieldName], {
				dirty: action.value !== state[action.formName].initial[action.fieldName],
				touched: true
			}))),
			values: _extends({}, state[action.formName].values, _defineProperty({}, action.fieldName, action.value))
		})));
	}

	/**
  * FIELD_VALIDATE_FAILURE
  *
  * Set a field's error
  *
  * @param {String} action.formName
  * @param {String} action.fieldName
  * @param {String} action.error
  */
	if (action.type === c.FIELD_VALIDATE_FAILURE) {
		return _extends({}, state, _defineProperty({}, action.formName, _extends({}, state[action.formName], {
			fields: _extends({}, state[action.formName].fields, _defineProperty({}, action.fieldName, _extends({}, state[action.formName].fields[action.fieldName], {
				error: action.error
			})))
		})));
	}

	/**
  * FIELD_VALIDATE_SUCCESS
  *
  * Clear a field's error
  *
  * @param {String} action.formName
  * @param {String} action.fieldName
  */
	if (action.type === c.FIELD_VALIDATE_SUCCESS) {
		return _extends({}, state, _defineProperty({}, action.formName, _extends({}, state[action.formName], {
			fields: _extends({}, state[action.formName].fields, _defineProperty({}, action.fieldName, _extends({}, state[action.formName].fields[action.fieldName], {
				error: null
			})))
		})));
	}

	return state;
}