'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = required;
/**
 * Create a new required validator
 *
 * @param {String} [message] - The message to use if validator fails. Defaults to "Value is required"
 *
 * @return {Function} - The validator function
 */
function required() {
	var message = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'Value is required';

	/**
  * Check if given value passes the required validator
  *
  * @param {String} value - The value to validate
  *
  * @return {Promise} - Gets rejected if validator fails, resolved otherwise
  */
	return function requiredValidator(value) {
		return new Promise(function (resolve, reject) {
			if (typeof value !== 'string') return reject('Given value is not a string');
			return value.replace(/\s+/, '') !== '' ? resolve(value) : reject(message);
		});
	};
}